@@ -283,7 +283,14 @@ class TileCreator(object):
         Query all sources and return the results as a single ImageSource.
         Multiple sources will be merged into a single image.
         """
-        if len(self.sources) == 1 and not self.image_merger:
+
+        # directly return get_map without merge if ...
+        if (len(self.sources) == 1 and
+            not self.image_merger and # no special image_merger (like BandMerger)
+            not (self.sources[0].coverage and  # no clipping coverage
+                 self.sources[0].coverage.clip and
+                 self.sources[0].coverage.intersects(query.bbox, query.srs))
+        ):
             try:
                 return self.sources[0].get_map(query)
             except BlankImage:
