@@ -621,14 +621,47 @@ class ArcGISSourceConfiguration(SourceConfiguration):
         request = create_request(self.conf["req"], params)
         http_client, request.url = self.http_client(request.url)
         coverage = self.coverage()
+        res_range = resolution_range(self.conf)
 
         client = ArcGISClient(request, http_client)
         image_opts = self.image_opts(format=params.get('format'))
         return ArcGISSource(client, image_opts=image_opts, coverage=coverage,
+                            res_range=res_range,
                             supported_srs=supported_srs,
                             supported_formats=supported_formats or None)
 
 
+    def fi_source(self, params=None):
+        from mapproxy.client.arcgis import ArcGISInfoClient
+        from mapproxy.request.arcgis import create_identify_request
+        from mapproxy.source.arcgis import ArcGISInfoSource
+        from mapproxy.srs import SRS
+
+        if params is None: params = {}
+        request_format = self.conf['req'].get('format')
+        if request_format:
+            params['format'] = request_format
+        supported_srs = [SRS(code) for code in self.conf.get('supported_srs', [])]
+        fi_source = None
+        if self.conf.get('opts', {}).get('featureinfo', False):
+            opts = self.conf['opts']
+            tolerance = opts.get('featureinfo_tolerance', 5)
+            return_geometries = opts.get('featureinfo_return_geometries', False)
+
+            fi_request = create_identify_request(self.conf['req'], params)
+
+
+            http_client, fi_request.url = self.http_client(fi_request.url)
+            fi_client = ArcGISInfoClient(fi_request,
+                supported_srs=supported_srs,
+                http_client=http_client,
+                tolerance=tolerance,
+                return_geometries=return_geometries,
+            )
+            fi_source = ArcGISInfoSource(fi_client)
+        return fi_source
+
+
 class WMSSourceConfiguration(SourceConfiguration):
     source_type = ('wms',)
 
@@ -953,6 +986,10 @@ class CacheConfiguration(ConfigurationBase):
         return self.context.globals.get_path('cache_dir', self.conf,
             global_key='cache.base_dir')
 
+    @memoize
+    def has_multiple_grids(self):
+        return len(self.grid_confs()) > 1
+
     def lock_dir(self):
         lock_dir = self.context.globals.get_path('cache.tile_lock_dir', self.conf)
         if not lock_dir:
@@ -965,6 +1002,11 @@ class CacheConfiguration(ConfigurationBase):
         cache_dir = self.cache_dir()
         directory_layout = self.conf.get('cache', {}).get('directory_layout', 'tc')
         if self.conf.get('cache', {}).get('directory'):
+            if self.has_multiple_grids():
+                raise ConfigurationError(
+                    "using single directory for cache with multiple grids in %s" %
+                    (self.conf['name']),
+                )
             pass
         elif self.conf.get('cache', {}).get('use_grid_names'):
             cache_dir = os.path.join(cache_dir, self.conf['name'], grid_conf.tile_grid().name)
@@ -978,13 +1020,10 @@ class CacheConfiguration(ConfigurationBase):
             log.warn('link_single_color_images not supported on windows')
             link_single_color_images = False
 
-        lock_timeout = self.context.globals.get_value('http.client_timeout', {})
-
         return FileCache(
             cache_dir,
             file_ext=file_ext,
             directory_layout=directory_layout,
-            lock_timeout=lock_timeout,
             link_single_color_images=link_single_color_images,
         )
 
@@ -1000,8 +1039,77 @@ class CacheConfiguration(ConfigurationBase):
         else:
             mbfile_path = os.path.join(self.cache_dir(), filename)
 
+        sqlite_timeout = self.context.globals.get_value('cache.sqlite_timeout', self.conf)
+        wal = self.context.globals.get_value('cache.sqlite_wal', self.conf)
+
         return MBTilesCache(
             mbfile_path,
+            timeout=sqlite_timeout,
+            wal=wal,
+        )
+
+    def _geopackage_cache(self, grid_conf, file_ext):
+        from mapproxy.cache.geopackage import GeopackageCache, GeopackageLevelCache
+
+        filename = self.conf['cache'].get('filename')
+        table_name = self.conf['cache'].get('table_name') or \
+                     "{}_{}".format(self.conf['name'], grid_conf.tile_grid().name)
+        levels = self.conf['cache'].get('levels')
+
+        if not filename:
+            filename = self.conf['name'] + '.gpkg'
+        if filename.startswith('.' + os.sep):
+            gpkg_file_path = self.context.globals.abspath(filename)
+        else:
+            gpkg_file_path = os.path.join(self.cache_dir(), filename)
+
+        cache_dir = self.conf['cache'].get('directory')
+        if cache_dir:
+            cache_dir = os.path.join(
+                self.context.globals.abspath(cache_dir),
+                grid_conf.tile_grid().name
+            )
+        else:
+            cache_dir = self.cache_dir()
+            cache_dir = os.path.join(
+                cache_dir,
+                self.conf['name'],
+                grid_conf.tile_grid().name
+            )
+
+        if levels:
+            return GeopackageLevelCache(
+                cache_dir, grid_conf.tile_grid(), table_name
+            )
+        else:
+            return GeopackageCache(
+                gpkg_file_path, grid_conf.tile_grid(), table_name
+            )
+
+    def _s3_cache(self, grid_conf, file_ext):
+        from mapproxy.cache.s3 import S3Cache
+
+        bucket_name = self.context.globals.get_value('cache.bucket_name', self.conf,
+            global_key='cache.s3.bucket_name')
+
+        if not bucket_name:
+            raise ConfigurationError("no bucket_name configured for s3 cache %s" % self.conf['name'])
+
+        profile_name = self.context.globals.get_value('cache.profile_name', self.conf,
+            global_key='cache.s3.profile_name')
+
+        directory_layout = self.conf['cache'].get('directory_layout', 'tms')
+
+        base_path = self.conf['cache'].get('directory', None)
+        if base_path is None:
+            base_path = os.path.join(self.conf['name'], grid_conf.tile_grid().name)
+
+        return S3Cache(
+            base_path=base_path,
+            file_ext=file_ext,
+            directory_layout=directory_layout,
+            bucket_name=bucket_name,
+            profile_name=profile_name,
         )
 
     def _sqlite_cache(self, grid_conf, file_ext):
@@ -1021,8 +1129,13 @@ class CacheConfiguration(ConfigurationBase):
                 grid_conf.tile_grid().name
             )
 
+        sqlite_timeout = self.context.globals.get_value('cache.sqlite_timeout', self.conf)
+        wal = self.context.globals.get_value('cache.sqlite_wal', self.conf)
+
         return MBTilesLevelCache(
             cache_dir,
+            timeout=sqlite_timeout,
+            wal=wal,
         )
 
     def _couchdb_cache(self, grid_conf, file_ext):
@@ -1074,6 +1187,46 @@ class CacheConfiguration(ConfigurationBase):
             use_secondary_index=use_secondary_index,
         )
 
+    def _redis_cache(self, grid_conf, file_ext):
+        from mapproxy.cache.redis import RedisCache
+
+        host = self.conf['cache'].get('host', '127.0.0.1')
+        port = self.conf['cache'].get('port', 6379)
+        db = self.conf['cache'].get('db', 0)
+        ttl = self.conf['cache'].get('default_ttl', 3600)
+
+        prefix = self.conf['cache'].get('prefix')
+        if not prefix:
+            prefix = self.conf['name'] + '_' + grid_conf.tile_grid().name
+
+        return RedisCache(
+            host=host,
+            port=port,
+            db=db,
+            prefix=prefix,
+            ttl=ttl,
+        )
+
+    def _compact_cache(self, grid_conf, file_ext):
+        from mapproxy.cache.compact import CompactCacheV1
+
+        cache_dir = self.cache_dir()
+        if self.conf.get('cache', {}).get('directory'):
+            if self.has_multiple_grids():
+                raise ConfigurationError(
+                    "using single directory for cache with multiple grids in %s" %
+                    (self.conf['name']),
+                )
+            pass
+        else:
+            cache_dir = os.path.join(cache_dir, self.conf['name'], grid_conf.tile_grid().name)
+
+        if self.conf['cache']['version'] != 1:
+            raise ConfigurationError("compact cache only supports version 1")
+        return CompactCacheV1(
+            cache_dir=cache_dir,
+        )
+
     def _tile_cache(self, grid_conf, file_ext):
         if self.conf.get('disable_storage', False):
             from mapproxy.cache.dummy import DummyCache
@@ -1228,7 +1381,7 @@ class CacheConfiguration(ConfigurationBase):
                     factor=source.get('factor', 1.0),
                 )
 
-        return band_merger.merge, sources, source_image_opts
+        return band_merger, sources, source_image_opts
 
     @memoize
     def caches(self):
@@ -1253,6 +1406,8 @@ class CacheConfiguration(ConfigurationBase):
             global_key='cache.meta_buffer')
         meta_size = self.context.globals.get_value('meta_size', self.conf,
             global_key='cache.meta_size')
+        bulk_meta_tiles = self.context.globals.get_value('bulk_meta_tiles', self.conf,
+            global_key='cache.bulk_meta_tiles')
         minimize_meta_requests = self.context.globals.get_value('minimize_meta_requests', self.conf,
             global_key='cache.minimize_meta_requests')
         concurrent_tile_creators = self.context.globals.get_value('concurrent_tile_creators', self.conf,
@@ -1336,7 +1491,9 @@ class CacheConfiguration(ConfigurationBase):
                 minimize_meta_requests=minimize_meta_requests,
                 concurrent_tile_creators=concurrent_tile_creators,
                 pre_store_filter=tile_filter,
-                tile_creator_class=tile_creator_class)
+                tile_creator_class=tile_creator_class,
+                bulk_meta_tiles=bulk_meta_tiles,
+            )
             extent = merge_layer_extents(sources)
             if extent.is_default:
                 extent = map_extent_from_grid(tile_grid)
@@ -1493,7 +1650,7 @@ class LayerConfiguration(ConfigurationBase):
         return dimensions
 
     @memoize
-    def tile_layers(self):
+    def tile_layers(self, grid_name_as_path=False):
         from mapproxy.service.tile import TileLayer
         from mapproxy.cache.dummy import DummyCache
 
@@ -1524,7 +1681,6 @@ class LayerConfiguration(ConfigurationBase):
         tile_layers = []
         for cache_name in sources:
             for grid, extent, cache_source in self.context.caches[cache_name].caches():
-
                 if dimensions and not isinstance(cache_source.cache, DummyCache):
                     # caching of dimension layers is not supported yet
                     raise ConfigurationError(
@@ -1535,8 +1691,11 @@ class LayerConfiguration(ConfigurationBase):
                 md = {}
                 md['title'] = self.conf['title']
                 md['name'] = self.conf['name']
-                md['name_path'] = (self.conf['name'], grid.srs.srs_code.replace(':', '').upper())
                 md['grid_name'] = grid.name
+                if grid_name_as_path:
+                    md['name_path'] = (md['name'], md['grid_name'])
+                else:
+                    md['name_path'] = (self.conf['name'], grid.srs.srs_code.replace(':', '').upper())
                 md['name_internal'] = md['name_path'][0] + '_' + md['name_path'][1]
                 md['format'] = self.context.caches[cache_name].image_opts().format
                 md['cache_name'] = cache_name
@@ -1612,11 +1771,9 @@ class ServiceConfiguration(ConfigurationBase):
     def tile_layers(self, conf, use_grid_names=False):
         layers = odict()
         for layer_name, layer_conf in iteritems(self.context.layers):
-            for tile_layer in layer_conf.tile_layers():
+            for tile_layer in layer_conf.tile_layers(grid_name_as_path=use_grid_names):
                 if not tile_layer: continue
                 if use_grid_names:
-                    # new style layer names are tuples
-                    tile_layer.md['name_path'] = (tile_layer.md['name'], tile_layer.md['grid_name'])
                     layers[tile_layer.md['name_path']] = tile_layer
                 else:
                     layers[tile_layer.md['name_internal']] = tile_layer
