@@ -19,10 +19,18 @@ from __future__ import with_statement
 import os
 from io import BytesIO
 from mapproxy.compat.image import Image, ImageDraw
-from mapproxy.image import ImageSource, ReadBufWrapper, is_single_color_image
-from mapproxy.image import peek_image_format
+from mapproxy.image import (
+    ImageSource,
+    BlankImageSource,
+    ReadBufWrapper,
+    is_single_color_image,
+    peek_image_format,
+    _make_transparent as make_transparent,
+    SubImageSource,
+    img_has_transparency,
+    quantize,
+)
 from mapproxy.image.merge import merge_images, BandMerger
-from mapproxy.image import _make_transparent as make_transparent, SubImageSource, img_has_transparency, quantize
 from mapproxy.image.opts import ImageOptions
 from mapproxy.image.tile import TileMerger, TileSplitter
 from mapproxy.image.transform import ImageTransformer
@@ -311,6 +319,17 @@ class TestLayerMerge(object):
             (10*10, (127, 127, 255, 255)),
         ])
 
+    def test_merge_L(self):
+        img1 = ImageSource(Image.new('RGBA', (10, 10), (255, 0, 255, 255)))
+        img2 = ImageSource(Image.new('L', (10, 10), 100))
+
+        # img2 overlays img1
+        result = merge_images([img1, img2], ImageOptions(transparent=True))
+        img = result.as_image()
+        assert_img_colors_eq(img, [
+            (10*10, (100, 100, 100, 255)),
+        ])
+
     def test_paletted_merge(self):
         if not hasattr(Image, 'FASTOCTREE'):
             raise SkipTest()
@@ -347,6 +366,16 @@ class TestLayerMerge(object):
         img = result.as_image()
         eq_(img.getpixel((0, 0)), (0, 255, 255))
 
+    def test_merge_rgb_with_transp(self):
+        img1 = ImageSource(Image.new('RGB', (10, 10), (255, 0, 255)))
+        raw = Image.new('RGB', (10, 10), (0, 255, 255))
+        raw.info = {'transparency': (0, 255, 255)} # make full transparent
+        img2 = ImageSource(raw)
+
+        result = merge_images([img1, img2], ImageOptions(transparent=False))
+        img = result.as_image()
+        eq_(img.getpixel((0, 0)), (255, 0, 255))
+
 
 class TestLayerCompositeMerge(object):
     def test_composite_merge(self):
@@ -582,6 +611,7 @@ class TestBandMerge(object):
         self.img1 = ImageSource(Image.new('RGB', (10, 10), (100, 110, 120)))
         self.img2 = ImageSource(Image.new('RGB', (10, 10), (200, 210, 220)))
         self.img3 = ImageSource(Image.new('RGB', (10, 10), (0, 255, 0)))
+        self.blank = BlankImageSource(size=(10, 10), image_opts=ImageOptions())
 
     def test_merge_noops(self):
         """
@@ -595,12 +625,15 @@ class TestBandMerge(object):
         eq_(img.size, (10, 10))
         eq_(img.getpixel((0, 0)), (0, 0, 0))
 
-    def test_merge_no_source(self):
+    def test_merge_missing_source(self):
         """
-        Check that empty source list returns BlankImageSource.
+        Check that empty source list or source list with missing images
+        returns BlankImageSource.
         """
         merger = BandMerger(mode='RGB')
         merger.add_ops(dst_band=0, src_img=0, src_band=0)
+        merger.add_ops(dst_band=1, src_img=1, src_band=0)
+        merger.add_ops(dst_band=2, src_img=2, src_band=0)
 
         img_opts = ImageOptions('RGBA', transparent=True)
         result = merger.merge([], img_opts, size=(10, 10))
@@ -609,6 +642,13 @@ class TestBandMerge(object):
         eq_(img.size, (10, 10))
         eq_(img.getpixel((0, 0)), (255, 255, 255, 0))
 
+        result = merger.merge([self.img0, self.img1], img_opts, size=(10, 10))
+        img = result.as_image()
+
+        eq_(img.size, (10, 10))
+        eq_(img.getpixel((0, 0)), (255, 255, 255, 0))
+
+
     def test_rgb_merge(self):
         """
         Check merge of RGB bands
